import 'package:furima_item_search/fs_error.dart';
import 'package:furima_item_search/fs_page_data.dart';
import 'package:furima_item_search/fs_item.dart';
import 'package:furima_item_search/fs_query.dart';
import 'package:furima_item_search/fs_site.dart';
import 'package:html/dom.dart';
import 'package:http/http.dart' as http;
import 'package:html/parser.dart' show parse;

class FSItemSearch {
  static Future<FSPageData> getPageData({FSQuery query, FSSite site}) async {
    final url = FSSiteHelper.getSearchPath(site, query);
    final FSPageData pageData = FSPageData(
        site: site, url: url, list: [], currentIndex: 0, hasNext: true);
    return getNextPage(pageData).then((data) {
      return data;
    });
  }

  static Future<FSPageData> getNextPage(FSPageData pageData) async {
    final String url = pageData.nextPageUrl;
    print(url);
    final response = await http.get(url);
    if (response.statusCode == 200) {
      final document = parse(response.body);
      final itemList = _parseItemList(site: pageData.site, document: document);
      if (itemList.length > 0) {
        pageData.list.addAll(itemList);
        pageData.currentIndex += 1;
      } else {
        pageData.hasNext = false;
      }
      return pageData;
    } else {
      throw (FSError.connectionError);
    }
  }

  static List<FSItem> _parseItemList({FSSite site, Document document}) {
    switch (site) {
      case FSSite.mercari:
        return _parseMercariItemList(document);
      case FSSite.rakuma:
        return _parseRakumaItemList(document);
      case FSSite.yahoo:
        return _parseYahooItemList(document);
    }
  }

  static List<FSItem> _parseMercariItemList(Document document) {
    final error = document.getElementsByClassName('search-result-description');
    if (error.isNotEmpty) {
      return [];
    }
    return document
        .getElementsByClassName('items-box-content')
        .first
        .getElementsByClassName('items-box')
        .map((e) {
      final title =
          e.getElementsByClassName('items-box-name').first.text.trim();
      final price =
          e.getElementsByClassName('items-box-price').first.text.trim();
      final thumbnailUrl = e
          .getElementsByClassName('items-box-photo')
          .first
          .getElementsByTagName('img')
          .first
          .attributes['data-src'];
      final linkUrl = e.getElementsByTagName('a').first.attributes['href'];
      final isSold = e.getElementsByClassName('item-sold-out-badge').isNotEmpty;
      return FSItem(
          title: title,
          price: price,
          thumbnailUrl: thumbnailUrl,
          linkUrl: linkUrl,
          isSold: isSold);
    }).toList();
  }

  static List<FSItem> _parseRakumaItemList(Document document) {
    return document
        .getElementsByClassName('view_grid')
        .first
        .getElementsByClassName('item')
        .map((e) {
      final title =
          e.getElementsByClassName('item-box__item-name').first.text.trim();
      final price =
          e.getElementsByClassName('item-box__item-price').first.text.trim();
      final thumbnailUrl = e
          .getElementsByClassName('item-box__image-wrapper')
          .first
          .getElementsByTagName('img')
          .first
          .attributes['data-original'];
      final linkUrl = e.getElementsByTagName('a').first.attributes['href'];
      final isSold =
          e.getElementsByClassName('item-box__soldout_ribbon').isNotEmpty;
      return FSItem(
          title: title,
          price: price,
          thumbnailUrl: thumbnailUrl,
          linkUrl: linkUrl,
          isSold: isSold);
    }).toList();
  }

  static List<FSItem> _parseYahooItemList(Document document) {
    final result = document.getElementsByClassName('Result').first;
    if (result == null) {
      throw (FSError.connectionError);
    }
    if (result.getElementsByClassName('Empty__title').isNotEmpty) {
      return [];
    }
    List<FSItem> list = result
        .getElementsByClassName('Products__items')
        .first
        .getElementsByClassName('Product')
        .map((e) {
      final title = e.getElementsByClassName('Product__title');
      final linkUrl = e.getElementsByTagName('a');
      if (title.isEmpty || linkUrl.isEmpty) {
        return null;
      }
      final price =
          e.getElementsByClassName('Product__priceValue').first?.text?.trim() ??
              "";
      final thumbnailUrl = e
          .getElementsByClassName('Product__imageData')
          .first
          .attributes['src'];

      return FSItem(
          title: title.first.text.trim(),
          price: price,
          thumbnailUrl: thumbnailUrl,
          linkUrl: linkUrl.first.attributes['href'],
          isSold: false);
    }).toList();
    list.removeWhere((value) => value == null);
    return list;
  }
}
